from typing import Dict, List

from ._core import mongodb

absendb = mongodb.absen


async def get_notes_count() -> dict:
    chats_count = 0
    absen_count = 0
    async for chat in absendb.find({"chat_id": {"$exists": 1}}):
        absen_name = await get_absen(chat["chat_id"])
        absen_count += len(absen_name)
        chats_count += 1
    return {"chats_count": chats_count, "absen_count": absen_count}


async def _get_absen(chat_id: int) -> Dict[str, int]:
    _absen = await absendb.find_one({"chat_id": chat_id})
    if not _absen:
        return {}
    return _absen["absen"]


async def get_absen(chat_id: int) -> List[str]:
    _absen = []
    for abse in await _get_absen(chat_id):
        _absen.append(abse)
    return _absen


async def save_absen(chat_id: int, name: str, asbes: dict):
    name = name.lower().strip()
    _absen = await _get_absen(chat_id)
    _absen[name] = asbes

    await absendb.update_one(
        {"chat_id": chat_id}, {"$set": {"absen": _absen}}, upsert=True
    )


async def del_absen(chat_id: int) -> bool:
    absensd = await _get_absen(chat_id)
    for name in absensd:
        del absensd[name]
        await absendb.update_one(
            {"chat_id": chat_id},
            {"$set": {"absen": absensd}},
            upsert=True,
        )
        return True
    return False



# py - Ayiin
# Copyright (C) 2022-2023 @AyiinXd
#
# This file is a part of < https://github.com/AyiinXd/pyAyiin >
# PLease read the GNU Affero General Public License in
# <https://www.github.com/AyiinXd/pyAyiin/blob/main/LICENSE/>.
#
# FROM py-Ayiin <https://github.com/AyiinXd/pyAyiin>
# t.me/AyiinChat & t.me/AyiinSupport


# ========================×========================
#            Jangan Hapus Credit Ngentod
# ========================×========================



class Queues(object):
    queue = {}

    def add_to_queue(self, chat_id, songname, link, ref, type, quality):
        if chat_id in self.queue:
            chat_queue = self.queue[chat_id]
            chat_queue.append([songname, link, ref, type, quality])
            return int(len(chat_queue) - 1)
        self.queue[chat_id] = [[songname, link, ref, type, quality]]
    
    def get_queue(self, chat_id):
        if chat_id in self.queue:
            return self.queue[chat_id]
        return 0
    
    def pop_an_item(self, chat_id):
        if chat_id in self.queue:
            chat_queue = self.queue[chat_id]
            chat_queue.pop(0)
            return 1
        return 0
    
    def clear_queue(self, chat_id: int):
        if chat_id in self.queue:
            self.queue.pop(chat_id)
            return 1
        return 0
    
    async def skip_song(self, client, chat_id):
        try:
            if chat_id in self.queue:
                chat_queue = self.get_queue(chat_id)
                if len(chat_queue) == 1:
                    await client.group_call.leave()
                    self.clear_queue(chat_id)
                    return 1
                else:
                    songname = chat_queue[1][0]
                    url = chat_queue[1][1]
                    link = chat_queue[1][2]
                    type = chat_queue[1][3]
                    chat_queue[1][4]
                    if type == "Audio":
                        await client.group_call.start_audio(
                            url,
                            repeat=False,
                        )
                    elif type == "Video":
                        await client.group_call.start_video(
                            url,
                            repeat=False,
                        )
                    self.pop_an_item(chat_id)
                    return [songname, link, type]
            else:
                return 0
        except Exception as e:
            print(e)

    async def skip_item(self, chat_id, h):
        if chat_id in self.queue:
            chat_queue = self.get_queue(chat_id)
            try:
                x = int(h)
                songname = chat_queue[x][0]
                chat_queue.pop(x)
                return songname
            except Exception as e:
                print(e)
                return 0
        else:
            return 0



# py - Ayiin
# Copyright (C) 2022-2023 @AyiinXd
#
# This file is a part of < https://github.com/AyiinXd/pyAyiin >
# PLease read the GNU Affero General Public License in
# <https://www.github.com/AyiinXd/pyAyiin/blob/main/LICENSE/>.
#
# FROM py-Ayiin <https://github.com/AyiinXd/pyAyiin>
# t.me/AyiinChat & t.me/AyiinSupport


# ========================×========================
#            Jangan Hapus Credit Ngentod
# ========================×========================


import asyncio
import os
import yt_dlp

from typing import Optional, Union

from fipper import Client
from fipper.types import Message
from fipper.raw.functions.channels import GetFullChannel
from fipper.raw.functions.messages import GetFullChat
from fipper.raw.functions.phone import CreateGroupCall, DiscardGroupCall, EditGroupCallTitle
from fipper.raw.types import InputGroupCall, InputPeerChannel, InputPeerChat

from pytgcalls.exceptions import GroupCallNotFoundError

from ..methods.queue import Queues

from .client import *


ACTIVE_CALLS, QUEUE = [], {}
MSGID_CACHE, PLAY_ON = {}, {}
CLIENTS = {}


class GroupCalls(object):
    async def get_group_call(
        self,
        client: Client, 
        message: Message, 
        err_msg: str = "",
    ) -> Optional[InputGroupCall]:
        chat_peer = await client.resolve_peer(message.chat.id)
        if isinstance(chat_peer, (InputPeerChannel, InputPeerChat)):
            if isinstance(chat_peer, InputPeerChannel):
                full_chat = (await client.invoke(GetFullChannel(channel=chat_peer))).full_chat
            elif isinstance(chat_peer, InputPeerChat):
                full_chat = (
                    await client.invoke(GetFullChat(chat_id=chat_peer.chat_id))
                ).full_chat
            if full_chat is not None:
                return full_chat.call
        await message.edit(f"<b>No group call Found</b> {err_msg}")
        return False
    
    async def TitleVc(self, client, m, title: str):
        peer = await client.resolve_peer(m.chat.id)
        if isinstance(peer, InputPeerChannel):
            chat = await client.invoke(GetFullChannel(channel=peer))
        if isinstance(peer, InputPeerChat):
            chat = await client.invoke(GetFullChat(chat_id=peer.chat_id))
        return await client.invoke(
            EditGroupCallTitle(
                call=chat.full_chat.call,
                title=title,
            )
        )

    async def StartVc(self, client, m, title=None):
        peer = await client.resolve_peer(m.chat.id)
        await client.invoke(
            CreateGroupCall(
                peer=InputPeerChannel(
                    channel_id=peer.channel_id,
                    access_hash=peer.access_hash,
                ),
                random_id=client.rnd_id() // 9000000000,
            )
        )
        titt = title if title else "🎧 Ayiin Music 🎧"
        await self.TitleVc(client, m, title=titt)

    async def StopVc(
        self,
        client,
        message,
    ):
        group_call = await self.get_group_call(client, message, err_msg="group call already ended")
        if not group_call:
            return
        await client.invoke(DiscardGroupCall(call=group_call))


class VcMusic(Queues):
    def __init__(self, client: Client, message: Message):
        self.m = message
        self.chat_id = []
        self.clients = {}
        self.client = client
        self.active_calls = []
        self.msgid_cache = {}
        self.play_on = {}

    async def startCall(self):
        if PLAY_ON:
            for chats in PLAY_ON:
                await PLAY_ON[chats].stop()
            PLAY_ON.clear()
            await asyncio.sleep(3)
        if self.m.video:
            for chats in list(CLIENTS):
                if chats != self.m.chat.id:
                    await CLIENTS[chats].stop()
                    del CLIENTS[chats]
            PLAY_ON.update({self.m.chat.id: self.client.group_call})
        if self.m.chat.id not in ACTIVE_CALLS:
            try:
                self.client.group_call.on_network_status_changed(
                    self.on_network_changed)
                self.client.group_call.on_playout_ended(self.playout_ended_handler)
                await self.client.group_call.join(self.m.chat.id)
            except GroupCallNotFoundError as er:
                await self.m.reply(er)
                dn, err = await self.StartVc()
                if err:
                    return False, err
            except Exception as e:
                await self.m.reply(e)
                return False, e
        return True, None

    async def on_network_changed(self, call, is_connected):
        chat = self.m.chat.id
        if is_connected:
            if chat not in ACTIVE_CALLS:
                ACTIVE_CALLS.append(chat)
        elif chat in ACTIVE_CALLS:
            ACTIVE_CALLS.remove(chat)

    async def playout_ended_handler(self, call, source, mtype):
        if os.path.exists(source):
            os.remove(source)
        await self.play_from_queue()

    async def play_from_queue(self):
        chat_id = self.m.chat.id
        if chat_id in PLAY_ON:
            await self.client.group_call.stop_video()
            PLAY_ON.pop(chat_id)
        try:
            ppk = await self.skip_song(self.client, chat_id)
            if ppk == 0:
                await self.client.group_call.leave()
            elif ppk == 1:
                await self.m.reply("<b>Antrian Udah Kosong Tod, Gua Cabut Dari OS</b>")
            else:
                await self.m.reply(
                    f"<b>• Memutar</b> - <a href='{ppk[1]}'>{ppk[0]}</a> | <code>{ppk[2]}</code>\n• <b>Pengguna:</b> {self.client.me.mention}",
                    disable_web_page_preview=True,
                )
        except Exception as e:
            await self.m.reply(e)


    async def JoinVc(self):
        chat_id = self.m.chat.id
        done, err = await self.startCall()

        if done:
            return True
        await self.client.send_message(
            self.m.chat.id,
            f"KESALAHAN saat Bergabung ke Obrolan Suara - {chat_id} :\n{err}",
        )
        return False



# AyiinXd
# Copyright (C) 2022-2023 @AyiinXd
#
# This file is a part of < https://github.com/AyiinXd/AyiinXd >
# PLease read the GNU Affero General Public License in
# <https://www.github.com/AyiinXd/AyiinXd/blob/main/LICENSE/>.
#
# FROM AyiinXd <https://github.com/AyiinXd/AyiinXd>
# t.me/AyiinChat & t.me/AyiinSupport

# ========================×========================
#            Jangan Hapus Credit Ngentod
# ========================×========================

import aiohttp
import asyncio
import importlib
import math
import os
import shlex
import textwrap
from typing import Tuple, Union
from io import BytesIO

from pymediainfo import MediaInfo
from fipper import Client, enums
from fipper.types import Message, User
from PIL import Image, ImageDraw, ImageFont


class Tools(object):
    async def CheckAdmin(
        self,
        client: Client,
        message: Message
    ):
        """Check if we are an admin."""
        admin = enums.ChatMemberStatus.ADMINISTRATOR
        creator = enums.ChatMemberStatus.OWNER
        ranks = [admin, creator]
    
        SELF = await client.get_chat_member(
            chat_id=message.chat.id, user_id=client.me.id
        )
    
        if SELF.status not in ranks:
            await message.reply_text("<i>Maaf Anda Bukan Admin!</i>")
    
        else:
            if SELF.status is admin or SELF.privileges:
                return True
            else:
                await message.reply_text("<i>Anda Tidak Mempunyai Izin Untuk Melakukan Itu</i>")
    
    
    def get_cmd(
        self,
        message: Message
    ):
        msg = message.text
        msg = msg.replace(" ", "", 1) if msg[1] == " " else msg
        split = msg[1:].replace("\n", " \n").split(" ")
        if " ".join(split[1:]).strip() == "":
            return ""
        return " ".join(split[1:])
    
    
    def get_args(
        self,
        message: Message
    ):
        try:
            message = message.text
        except AttributeError:
            pass
        if not message:
            return False
        message = message.split(maxsplit=1)
        if len(message) <= 1:
            return []
        message = message[1]
        try:
            split = shlex.split(message)
        except ValueError:
            return message
        return list(filter(lambda x: len(x) > 0, split))
    
    
    def get_user(
        self,
        message: Message,
        text: str
    ) -> Tuple[int, str, None]:
        """Get User From Message"""
        if text is None:
            asplit = None
        else:
            asplit = text.split(" ", 1)
        user_s = None
        reason_ = None
        if message.reply_to_message:
            user_s = message.reply_to_message.from_user.id
            reason_ = text if text else None
        elif asplit is None:
            return None, None
        elif len(asplit[0]) > 0:
            if message.entities:
                if len(message.entities) == 1:
                    required_entity = message.entities[0]
                    if required_entity.type == "text_mention":
                        user_s = int(required_entity.user.id)
                    else:
                        user_s = int(
                            asplit[0]) if asplit[0].isdigit() else asplit[0]
            else:
                user_s = int(asplit[0]) if asplit[0].isdigit() else asplit[0]
            if len(asplit) == 2:
                reason_ = asplit[1]
        return user_s, reason_
    
    
    def get_text(
        self,
        message: Message
    ) -> Tuple[None, str]:
        """Extract Text From Commands"""
        text_to_return = message.text
        if message.text is None:
            return None
        if " " in text_to_return:
            try:
                return message.text.split(None, 1)[1]
            except IndexError:
                return None
        else:
            return None
    
    
    def media_data(
        self,
        media: str
    ) -> dict:
        "Get downloaded media's information"
        found = False
        media_info = MediaInfo.parse(media)
        for track in media_info.tracks:
            if track.track_type == "Video":
                found = True
                type_ = track.track_type
                format_ = track.format
                duration_1 = track.duration
                other_duration_ = track.other_duration
                duration_2 = (
                    f"{other_duration_[0]} - ({other_duration_[3]})"
                    if other_duration_
                    else None
                )
                pixel_ratio_ = [track.width, track.height]
                aspect_ratio_1 = track.display_aspect_ratio
                other_aspect_ratio_ = track.other_display_aspect_ratio
                aspect_ratio_2 = other_aspect_ratio_[
                    0] if other_aspect_ratio_ else None
                fps_ = track.frame_rate
                fc_ = track.frame_count
                media_size_1 = track.stream_size
                other_media_size_ = track.other_stream_size
                media_size_2 = (
                    [
                        other_media_size_[1],
                        other_media_size_[2],
                        other_media_size_[3],
                        other_media_size_[4],
                    ]
                    if other_media_size_
                    else None
                )

        dict_ = (
            {
                "media_type": type_,
                "format": format_,
                "duration_in_ms": duration_1,
                "duration": duration_2,
                "pixel_sizes": pixel_ratio_,
                "aspect_ratio_in_fraction": aspect_ratio_1,
                "aspect_ratio": aspect_ratio_2,
                "frame_rate": fps_,
                "frame_count": fc_,
                "file_size_in_bytes": media_size_1,
                "file_size": media_size_2,
            }
            if found
            else None
        )
        return dict_
    
    
    def resize_image(self, image):
        im = Image.open(image)
        maxsize = (512, 512)
        if (im.width and im.height) < 512:
            size1 = im.width
            size2 = im.height
            if im.width > im.height:
                scale = 512 / size1
                size1new = 512
                size2new = size2 * scale
            else:
                scale = 512 / size2
                size1new = size1 * scale
                size2new = 512
            size1new = math.floor(size1new)
            size2new = math.floor(size2new)
            sizenew = (size1new, size2new)
            im = im.resize(sizenew)
        else:
            im.thumbnail(maxsize)
        file_name = "Sticker.png"
        im.save(file_name, "PNG")
        if os.path.exists(image):
            os.remove(image)
        return file_name
    
    
    async def resize_media(
        self,
        media: str,
        video: bool,
        fast_forward: bool
    ) -> str:
        if video:
            info_ = self.media_data(media)
            width = info_["pixel_sizes"][0]
            height = info_["pixel_sizes"][1]
            sec = info_["duration_in_ms"]
            s = round(float(sec)) / 1000
    
            if height == width:
                height, width = 512, 512
            elif height > width:
                height, width = 512, -1
            elif width > height:
                height, width = -1, 512
    
            resized_video = f"{media}.webm"
            if fast_forward:
                if s > 3:
                    fract_ = 3 / s
                    ff_f = round(fract_, 2)
                    set_pts_ = ff_f - 0.01 if ff_f > fract_ else ff_f
                    cmd_f = f"-filter:v 'setpts={set_pts_}*PTS',scale={width}:{height}"
                else:
                    cmd_f = f"-filter:v scale={width}:{height}"
            else:
                cmd_f = f"-filter:v scale={width}:{height}"
            fps_ = float(info_["frame_rate"])
            fps_cmd = "-r 30 " if fps_ > 30 else ""
            cmd = f"ffmpeg -i {media} {cmd_f} -ss 00:00:00 -to 00:00:03 -an -c:v libvpx-vp9 {fps_cmd}-fs 256K {resized_video}"
            _, error, __, ___ = await self.run_cmd(cmd)
            os.remove(media)
            return resized_video
    
        image = Image.open(media)
        maxsize = 512
        scale = maxsize / max(image.width, image.height)
        new_size = (int(image.width * scale), int(image.height * scale))
    
        image = image.resize(new_size, Image.LANCZOS)
        resized_photo = "sticker.png"
        image.save(resized_photo)
        os.remove(media)
        return resized_photo



import logging

from asyncio import sleep

from telethon.errors import MessageDeleteForbiddenError, MessageNotModifiedError
from telethon.tl.custom import Message
from telethon.tl.types import MessageService

# edit or reply

LOGS = logging.getLogger("pyAyiin")


async def eor(event, text=None, **args):
    time = args.get("time", None)
    edit_time = args.get("edit_time", None)
    if "edit_time" in args:
        del args["edit_time"]
    if "time" in args:
        del args["time"]
    if "link_preview" not in args:
        args["link_preview"] = False
    args["reply_to"] = event.reply_to_msg_id or event
    if event.out and not isinstance(event, MessageService):
        if edit_time:
            await sleep(edit_time)
        if "file" in args and args["file"] and not event.media:
            await event.delete()
            ok = await event.client.send_message(event.chat_id, text, **args)
        else:
            try:
                try:
                    del args["reply_to"]
                except KeyError:
                    pass
                ok = await event.edit(text, **args)
            except MessageNotModifiedError:
                ok = event
    else:
        ok = await event.client.send_message(event.chat_id, text, **args)

    if time:
        await sleep(time)
        return await ok.delete()
    return ok


async def eod(event, text=None, **kwargs):
    kwargs["time"] = kwargs.get("time", 8)
    return await eor(event, text, **kwargs)


async def _try_delete(event):
    try:
        return await event.delete()
    except (MessageDeleteForbiddenError):
        pass
    except BaseException as er:
        LOGS.error("Error while Deleting Message..")
        LOGS.exception(er)


setattr(Message, "eor", eor)
setattr(Message, "try_delete", _try_delete)



from typing import List

from ._core import mongodb

blacklist_filtersdb = mongodb.blacklistfilter


async def get_blacklist_filters_count() -> dict:
    chats_count = 0
    filters_count = 0
    async for chat in blacklist_filtersdb.find({"chat_id": {"$lt": 0}}):
        filters = await get_blacklisted_words(chat["chat_id"])
        filters_count += len(filters)
        chats_count += 1
    return {
        "chats_count": chats_count,
        "filters_count": filters_count,
    }


async def get_blacklisted_words(chat_id: int) -> List[str]:
    _filters = await blacklist_filtersdb.find_one({"chat_id": chat_id})
    if not _filters:
        return []
    return _filters["filters"]


async def save_blacklist_filter(chat_id: int, word: str):
    word = word.lower().strip()
    _filters = await get_blacklisted_words(chat_id)
    _filters.append(word)
    await blacklist_filtersdb.update_one(
        {"chat_id": chat_id},
        {"$set": {"filters": _filters}},
        upsert=True,
    )


async def delete_blacklist_filter(chat_id: int, word: str) -> bool:
    filtersd = await get_blacklisted_words(chat_id)
    word = word.lower().strip()
    if word in filtersd:
        filtersd.remove(word)
        await blacklist_filtersdb.update_one(
            {"chat_id": chat_id},
            {"$set": {"filters": filtersd}},
            upsert=True,
        )
        return True
    return False
